@@ -1,3 +1,5 @@
+# -*- indent-tabs-mode: nil -*-
+
 package CPAN::Mini::Inject::Remote;
 
 use strict;
@@ -18,11 +20,11 @@ CPAN::Mini::Inject::Remote - Inject into your CPAN mirror from over here
 
 =head1 VERSION
 
-Version 0.02
+Version 0.04
 
 =cut
 
-our $VERSION = '0.02';
+our $VERSION = '0.04';
 
 
 =head1 SYNOPSIS
@@ -77,20 +79,20 @@ sub _initialize {
         }
     );
 
+    if (not $args{config_file})
+    {
+	$args{config_file} = $self->_find_config();
+    }
+    elsif (not -r $args{config_file})
+    {
+	croak "Supplied config file is not readable";
+    }
+
+    my $config = LoadFile($args{config_file});
+
     if (not $args{remote_server})
     {
-        if (not $args{config_file})
-        {
-            $args{config_file} = $self->_find_config();
-        }
-        elsif (not -r $args{config_file})
-        {
-            croak "Supplied config file is not readable"; 
-        }
-
-        my $config = LoadFile($args{config_file}); 
-
-        $self->{remote_server} = $config->{remote_server};
+	$self->{remote_server} = $config->{remote_server};
     }
     else
     {
@@ -100,6 +102,25 @@ sub _initialize {
     # get rid of any trailing slash as it will break things
     $self->{remote_server} =~ s/\/$//;
 
+    my @ssl_opt = qw/SSL_ca_file SSL_cert_file SSL_key_file verify_hostnames/;
+    for (@ssl_opt)
+    {
+        next unless my $c = $config->{$_};
+        if ($c =~ s/^\s*#!//)
+        {
+            my $output = eval { `$c` };
+            $self->{ssl_opts}{$_} = $output if $? == 0;
+        }
+        elsif ($c =~ /^~/)
+        {
+            $self->{ssl_opts}{$_} = (glob $c)[0];
+        }
+        else
+        {
+            $self->{ssl_opts}{$_} = $c;
+        }
+    }
+
 } # end of method _initialize
 
 
@@ -173,6 +194,11 @@ sub _useragent {
     if (not $self->{useragent})
     {
         $self->{useragent} = LWP::UserAgent->new;
+
+        if ($self->{remote_server} =~ /^https:/ && $self->{ssl_opts})
+        {
+            $self->{useragent}->ssl_opts(%{$self->{ssl_opts}});
+        }
     }
 
     return $self->{useragent};
@@ -224,10 +250,11 @@ sub add {
 
     if (not $response->is_success())
     {
-        croak 'Add failed. ' . Dumper($response);
+        #croak 'Add failed. ' . Dumper($response);
+        warn 'Add failed. ' . $response->status_line . "\n";
     }
 
-    return;
+    return $response;
 } # end of method add
 
 
@@ -251,10 +278,11 @@ sub update {
 
     if (not $response->is_success())
     {
-        croak 'Update failed. ' . Dumper($response);
+        #croak 'Update failed. ' . Dumper($response);
+        warn 'Update failed. ' . $response->status_line . "\n";
     }
 
-    return;
+    return $response;
 } # end of method update
 
 
@@ -278,13 +306,34 @@ sub inject {
 
     if (not $response->is_success())
     {
-        croak 'Inject failed. ' . Dumper($response);
+        #croak 'Inject failed. ' . Dumper($response);
+        warn 'Inject failed. ' . $response->status_line . "\n";
     }
 
-    return;
+    return $response;
 } # end of method inject
 
 
+=head1 CONFIGURATION FILE
+
+the sample configuration file ~/.mcpani_remote over SSL:
+
+ remote_server: https://mcpani.your.org
+ SSL_cert_file: ~/.certs/your.crt
+ SSL_key_file: ~/.certs/your.key
+ SSL_ca_file: #!perl -MCACertOrg::CA -e 'print CACertOrg::CA::SSL_ca_file()'
+
+
+you want to export your.crt and your.key from your.p12:
+
+ $ openssl pkcs12 -nokeys -clcerts -in your.p12 -out your.crt
+ Enter Import Password: ******
+ MAC verified OK
+ $ openssl pkcs12 -nocerts -nodes -in your.p12 -out your.key
+ Enter Import Password: ******
+ MAC verified OK
+
+
 =head1 AUTHOR
 
 Christopher Mckay, C<< <potatohead at potatolan.com> >>
